package com.hichina.admin.hichinaadminbackend.checker;

import java.util.HashMap;
import java.util.Map;

public class ProductAttributeCheckerFactory {

    private static ProductAttributeCheckerFactory instance;

    static {
        instance = new ProductAttributeCheckerFactory();
        instance.addDataTypeChecker("string", new StringProductAttributeChecker());
        instance.addDataTypeChecker("image", new StringProductAttributeChecker());
        instance.addDataTypeChecker("datestring", new StringProductAttributeChecker());
        instance.addDataTypeChecker("integer", new IntProductAttributeChecker());
        instance.addDataTypeChecker("date", new DateProductAttributeChecker());
    }

    /**
     * attribute id -> checker
     */
    private Map<String, ProductAttributeChecker<?>> attributeIdCheckerMap;

    private Map<String, ProductAttributeChecker<?>> dataTypeChekcerMap;

    public ProductAttributeCheckerFactory() {
        this.attributeIdCheckerMap = new HashMap<>();
        this.dataTypeChekcerMap = new HashMap<>();
    }

    public static ProductAttributeCheckerFactory getInstance() {
        return instance;
    }

    public void addAttributeIdChecker(String attributeId, ProductAttributeChecker<?> checker) {
        this.attributeIdCheckerMap.put(attributeId, checker);
    }

    public void addDataTypeChecker(String dataType, ProductAttributeChecker<?> checker) {
        this.dataTypeChekcerMap.put(dataType, checker);
    }

    public ProductAttributeChecker<?> getAttributeChecker(String attributeId, String dateType) {
        ProductAttributeChecker<?> checker = attributeIdCheckerMap.get(attributeId);
        if (checker != null) {
            return checker;
        }

        checker = dataTypeChekcerMap.get(dateType);
        if (checker != null) {
            return checker;
        }
        throw new RuntimeException(String.format("unsupported attribute: %s [%s]", attributeId, dateType));
    }

    @SuppressWarnings("unchecked")
    public <T> T checkAndConvert(String attributeId, String dateType, String attributeValue) throws InvalidProductAttributeException {
        ProductAttributeChecker<?> checker = getAttributeChecker(attributeId, dateType);
        return (T) checker.check(attributeValue);
    }

    public void check(String attributeId, String dateType, String attributeValue) throws InvalidProductAttributeException {
        ProductAttributeChecker<?> checker = getAttributeChecker(attributeId, dateType);
        checker.check(attributeValue);
    }

}
