package com.hichina.admin.hichinaadminbackend.controller

import com.fasterxml.jackson.databind.ObjectMapper
import com.hichina.admin.hichinaadminbackend.HichinaAdminBackendApplication
import com.hichina.admin.hichinaadminbackend.config.Constants
import com.hichina.admin.hichinaadminbackend.mapper.*
import com.hichina.admin.hichinaadminbackend.model.DTO.ProductSkuBatchCreateRequest
import com.hichina.admin.hichinaadminbackend.model.ProductSkuGroup
import org.mybatis.spring.boot.test.autoconfigure.AutoConfigureMybatis
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.data.mongo.AutoConfigureDataMongo
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.security.test.context.support.WithMockUser
import org.springframework.test.context.ActiveProfiles
import org.springframework.test.web.servlet.MockMvc
import spock.lang.Specification

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status

@SpringBootTest(classes= HichinaAdminBackendApplication)
@ActiveProfiles("test")
@AutoConfigureMockMvc
@AutoConfigureMybatis
@AutoConfigureDataMongo
@WithMockUser(username = "admin", roles = ["ADMIN"])
@WebMvcTest
class ProductSkuSpec extends Specification {

    @Autowired
    MockMvc mvc

    @Autowired
    ProductSkuGroupMapper groupMapper

    @Autowired
    HichinaProductMapper productMapper

    @Autowired
    ProductSkuIntAttributeMappingMapper productSkuIntAttributeMappingMapper

    @Autowired
    ProductSkuVarcharAttributeMappingMapper productSkuVarcharAttributeMappingMapper

    @Autowired
    ProductSkuTimestampAttributeMappingMapper productSkuTimestampAttributeMappingMapper

    def "should batch add product skus if sku group not exist"() {
        given:
        def request = new ProductSkuBatchCreateRequest(
                productName: "test",
                productDescription: "test",
                productTypeId: Constants.TOURPRODUCTTYPE,
                comboCustomPropertyBags: [
                        [
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100"   : "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82"    : "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b"    : "test",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54"   : "1000",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1"   : "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404"   : "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b"   : "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5"    : "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492": "2024/3/20;2024/3/21;",
                        ],
                        [
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100"   : "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82"    : "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b"    : "test2",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54"   : "1200",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1"   : "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404"   : "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b"   : "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5"    : "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492": "2024/3/20;2024/3/21;",
                        ]
                ]
        )

        when:
        mvc.perform {
            post("/product/sku/batch")
                    .contentType("application/json")
                    .content((new ObjectMapper()).writeValueAsString(request))
        }

        then:
        1 * groupMapper.insert(_)
        2 * productMapper.insert(_)
        10 * productSkuIntAttributeMappingMapper.insert(_)
        8 * productSkuVarcharAttributeMappingMapper.insert(_)
        1 * groupMapper.updateMinPriceAndImage(_)
    }

    def "should batch add product skus if sku group exist"() {
        given:
        groupMapper.findByNameAndProductTypeId("test", Constants.TOURPRODUCTTYPE) >> [
                new ProductSkuGroup(skuGroupId: "1d4befa4-acf9-4de4-a596-2163cd45ea21", skuGroupName: "test")
        ]
        def request = new ProductSkuBatchCreateRequest(
                productName: "test",
                productDescription: "test",
                productTypeId: Constants.TOURPRODUCTTYPE,
                comboCustomPropertyBags: [
                        [
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100"   : "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82"    : "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b"    : "test",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54"   : "1000",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1"   : "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404"   : "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b"   : "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5"    : "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492": "2024/3/20;2024/3/21;",
                        ],
                        [
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100"   : "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82"    : "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b"    : "test2",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54"   : "1200",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1"   : "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404"   : "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b"   : "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5"    : "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492": "2024/3/20;2024/3/21;",
                        ]
                ]
        )

        when:
        mvc.perform {
            post("/product/sku/batch")
                    .contentType("application/json")
                    .content((new ObjectMapper()).writeValueAsString(request))
        }

        then:
        0 * groupMapper.insert(_)
        2 * productMapper.insert(_)
        10 * productSkuIntAttributeMappingMapper.insert(_)
        8 * productSkuVarcharAttributeMappingMapper.insert(_)
        1 * groupMapper.updateMinPriceAndImage(_)
    }

    def "should not batch add product skus if any sku is invalid"() {
        given:
        def request = new ProductSkuBatchCreateRequest(
                productName: "",
                productDescription: "test",
                productTypeId: Constants.TOURPRODUCTTYPE,
                comboCustomPropertyBags: [
                        [
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100"   : "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82"    : "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b"    : "test",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54"   : "1000",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1"   : "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404"   : "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b"   : "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5"    : "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492": "2024/3/20;2024/3/21;",
                        ],
                        [
                                "[integer]8cc865ff-b30f-4f00-b426-9e64418e5100"   : "1",
                                "[string]04a54b65-a1f3-45d9-955c-40cf88f21b82"    : "vendor",
                                "[string]11cd8b32-c4f6-47db-8b8a-486c992bf43b"    : "test2",
                                "[integer]e228b843-e054-41f8-91dd-19663460df54"   : "1200",
                                "[integer]c4c845a7-4bef-46d8-a5ad-d72a5464e8b1"   : "500",
                                "[integer]448406cb-b68f-439e-9da8-148d78ae8404"   : "0",
                                "[integer]d4825026-3ad8-4861-8e8f-e980b55fe67b"   : "200",
                                "[string]f3cac6ed-8e87-4fb9-a912-bd87f483b4d5"    : "testdesc1",
                                "[datestring]f0b807e5-d1a6-4454-a400-7905a4fea492": "2024/3/20;2024/3/21;",
                        ]
                ]
        )

        when:
        def result = mvc.perform {
            post("/api/v1/productsku/batch")
                    .contentType("application/json")
                    .content((new ObjectMapper()).writeValueAsString(request))
        }

        then:
        result.andExpect {
            status().isBadRequest()
        }
        1 == 1

    }
}